@@ -3,22 +3,27 @@ from typing import List
 
 from pydantic import BaseModel, SecretStr
 
+
 class Country(BaseModel):
     name: str
     phone_code: int
 
+
 class Address(BaseModel):
     post_code: int
     country: Country
 
+
 class CardDetails(BaseModel):
     number: SecretStr
     expires: datetime.date
 
+
 class Hobby(BaseModel):
     name: str
     info: str
 
+
 class User(BaseModel):
     first_name: str
     second_name: str
@@ -26,6 +31,7 @@ class User(BaseModel):
     card_details: CardDetails
     hobbies: List[Hobby]
 
+
 user = User(
     first_name='John',
     second_name='Doe',
@@ -42,24 +48,26 @@ user = User(
     ),
     hobbies=[
         Hobby(name='Programming', info='Writing code and stuff'),
-        Hobby(name='Gaming', info='Hell Yeah!!!')
-    ]
-
+        Hobby(name='Gaming', info='Hell Yeah!!!'),
+    ],
 )
 
 exclude_keys = {
-    'second_name': ...,
-    'address': {'post_code': ..., 'country': {'phone_code'}},
-    'card_details': ...,
-    # You can exclude values from tuples and lists by indexes
+    'second_name': True,
+    'address': {'post_code': True, 'country': {'phone_code'}},
+    'card_details': True,
+    # You can exclude fields from specific members of a tuple/list by index:
     'hobbies': {-1: {'info'}},
 }
 
 include_keys = {
-    'first_name': ...,
+    'first_name': True,
     'address': {'country': {'name'}},
-    'hobbies': {0: ..., -1: {'name'}}
+    'hobbies': {0: True, -1: {'name'}},
 }
 
 # would be the same as user.dict(exclude=exclude_keys) in this case:
 print(user.dict(include=include_keys))
+
+# To exclude a field from all members of a nested list or tuple, use "__all__":
+print(user.dict(exclude={'hobbies': {'__all__': {'info'}}}))
