@@ -1,18 +1,22 @@
-from typing import List
+from typing import List, Optional
 from pydantic import BaseModel
 
+
 class Foo(BaseModel):
     count: int
-    size: float = None
+    size: Optional[float] = None
+
 
 class Bar(BaseModel):
     apple = 'x'
     banana = 'y'
 
+
 class Spam(BaseModel):
     foo: Foo
     bars: List[Bar]
 
+
 m = Spam(foo={'count': 4}, bars=[{'apple': 'x1'}, {'apple': 'x2'}])
 print(m)
 print(m.dict())
