@@ -1,19 +1,28 @@
 from typing import Set
 
 from pydantic import (
-    BaseModel, BaseSettings, PyObject, RedisDsn, PostgresDsn, Field
+    BaseModel,
+    BaseSettings,
+    PyObject,
+    RedisDsn,
+    PostgresDsn,
+    AmqpDsn,
+    Field,
 )
 
+
 class SubModel(BaseModel):
     foo = 'bar'
     apple = 1
 
+
 class Settings(BaseSettings):
     auth_key: str
     api_key: str = Field(..., env='my_api_key')
 
     redis_dsn: RedisDsn = 'redis://user:pass@localhost:6379/1'
     pg_dsn: PostgresDsn = 'postgres://user:pass@localhost:5432/foobar'
+    amqp_dsn: AmqpDsn = 'amqp://user:pass@localhost:5672/'
 
     special_function: PyObject = 'math.cos'
 
@@ -36,4 +45,5 @@ class Settings(BaseSettings):
             }
         }
 
+
 print(Settings().dict())
