@@ -1,5 +1,6 @@
 from pydantic import BaseModel, ValidationError, validator
 
+
 class UserModel(BaseModel):
     name: str
     username: str
@@ -20,14 +21,24 @@ class UserModel(BaseModel):
 
     @validator('username')
     def username_alphanumeric(cls, v):
-        assert v.isalpha(), 'must be alphanumeric'
+        assert v.isalnum(), 'must be alphanumeric'
         return v
 
-print(UserModel(name='samuel colvin', username='scolvin', password1='zxcvbn',
-                password2='zxcvbn'))
+
+user = UserModel(
+    name='samuel colvin',
+    username='scolvin',
+    password1='zxcvbn',
+    password2='zxcvbn',
+)
+print(user)
 
 try:
-    UserModel(name='samuel', username='scolvin', password1='zxcvbn',
-              password2='zxcvbn2')
+    UserModel(
+        name='samuel',
+        username='scolvin',
+        password1='zxcvbn',
+        password2='zxcvbn2',
+    )
 except ValidationError as e:
     print(e)
