@@ -1,6 +1,16 @@
-__all__ = 'VERSION', 'version_info'
+__all__ = 'compiled', 'VERSION', 'version_info'
 
-VERSION = '1.7.4'
+VERSION = '1.10.4'
+
+try:
+    import cython  # type: ignore
+except ImportError:
+    compiled: bool = False
+else:  # pragma: no cover
+    try:
+        compiled = cython.compiled
+    except AttributeError:
+        compiled = False
 
 
 def version_info() -> str:
@@ -9,10 +19,8 @@ def version_info() -> str:
     from importlib import import_module
     from pathlib import Path
 
-    from .main import compiled
-
     optional_deps = []
-    for p in ('typing-extensions', 'email-validator', 'devtools'):
+    for p in ('devtools', 'dotenv', 'email-validator', 'typing-extensions'):
         try:
             import_module(p.replace('-', '_'))
         except ImportError:
