@@ -1472,3 +1472,139 @@ def test_subclass_post_init_inheritance():
             self.a *= 3
 
     assert C().a == 6  # 1 * 3 + 3
+
+
+def test_inheritance_post_init_2():
+    post_init_calls = 0
+    post_init_post_parse_calls = 0
+
+    @pydantic.dataclasses.dataclass
+    class BaseClass:
+        def __post_init__(self):
+            nonlocal post_init_calls
+            post_init_calls += 1
+
+    @pydantic.dataclasses.dataclass
+    class AbstractClass(BaseClass):
+        pass
+
+    @pydantic.dataclasses.dataclass
+    class ConcreteClass(AbstractClass):
+        def __post_init_post_parse__(self):
+            nonlocal post_init_post_parse_calls
+            post_init_post_parse_calls += 1
+
+    ConcreteClass()
+    assert post_init_calls == 1
+    assert post_init_post_parse_calls == 1
+
+
+def test_dataclass_setattr():
+    class Foo:
+        bar: str = 'cat'
+
+    default_config = dataclasses.make_dataclass(
+        cls_name=Foo.__name__,
+        bases=(dataclasses.dataclass(Foo),),
+        fields=[('bar', ClassVar[str], dataclasses.field(default=Foo.bar))],
+    )
+
+    config = pydantic.dataclasses.dataclass(default_config)
+    assert config.bar == 'cat'
+    setattr(config, 'bar', 'dog')
+    assert config.bar == 'dog'
+
+
+def test_frozen_dataclasses():
+    @dataclasses.dataclass(frozen=True)
+    class First:
+        a: int
+
+    @dataclasses.dataclass(frozen=True)
+    class Second(First):
+        @property
+        def b(self):
+            return self.a
+
+    class My(BaseModel):
+        my: Second
+
+    assert My(my=Second(a='1')).my.b == 1
+
+
+def test_empty_dataclass():
+    """should be able to inherit without adding a field"""
+
+    @dataclasses.dataclass
+    class UnvalidatedDataclass:
+        a: int = 0
+
+    @pydantic.dataclasses.dataclass
+    class ValidatedDerivedA(UnvalidatedDataclass):
+        ...
+
+    @pydantic.dataclasses.dataclass()
+    class ValidatedDerivedB(UnvalidatedDataclass):
+        b: int = 0
+
+    @pydantic.dataclasses.dataclass()
+    class ValidatedDerivedC(UnvalidatedDataclass):
+        ...
+
+
+def test_proxy_dataclass():
+    @dataclasses.dataclass
+    class Foo:
+        a: Optional[int] = dataclasses.field(default=42)
+        b: List = dataclasses.field(default_factory=list)
+
+        @dataclasses.dataclass
+        class Bar:
+            pass
+
+    @dataclasses.dataclass
+    class Model1:
+        foo: Foo
+
+    class Model2(BaseModel):
+        foo: Foo
+
+    m1 = Model1(foo=Foo())
+    m2 = Model2(foo=Foo())
+
+    assert m1.foo.a == m2.foo.a == 42
+    assert m1.foo.b == m2.foo.b == []
+    assert m1.foo.Bar() is not None
+    assert m2.foo.Bar() is not None
+
+
+def test_proxy_dataclass_2():
+    @dataclasses.dataclass
+    class M1:
+        a: int
+        b: str = 'b'
+        c: float = dataclasses.field(init=False)
+
+        def __post_init__(self):
+            self.c = float(self.a)
+
+    @dataclasses.dataclass
+    class M2:
+        a: int
+        b: str = 'b'
+        c: float = dataclasses.field(init=False)
+
+        def __post_init__(self):
+            self.c = float(self.a)
+
+        @pydantic.validator('b')
+        def check_b(cls, v):
+            if not v:
+                raise ValueError('b should not be empty')
+            return v
+
+    m1 = pydantic.parse_obj_as(M1, {'a': 3})
+    m2 = pydantic.parse_obj_as(M2, {'a': 3})
+    assert m1.a == m2.a == 3
+    assert m1.b == m2.b == 'b'
+    assert m1.c == m2.c == 3.0
