@@ -7,6 +7,7 @@ from typing import (
     ClassVar,
     Dict,
     Generic,
+    Iterable,
     List,
     Mapping,
     Optional,
@@ -23,10 +24,7 @@ from typing_extensions import Annotated, Literal
 from pydantic import BaseModel, Field, Json, ValidationError, root_validator, validator
 from pydantic.generics import GenericModel, _generic_types_cache, iter_contained_typevars, replace_types
 
-skip_36 = pytest.mark.skipif(sys.version_info < (3, 7), reason='generics only supported for python 3.7 and above')
 
-
-@skip_36
 def test_generic_name():
     data_type = TypeVar('data_type')
 
@@ -39,7 +37,6 @@ def test_generic_name():
     assert Result[int].__name__ == 'Result[int]'
 
 
-@skip_36
 def test_double_parameterize_error():
     data_type = TypeVar('data_type')
 
@@ -52,7 +49,6 @@ def test_double_parameterize_error():
     assert str(exc_info.value) == 'Cannot parameterize a concrete instantiation of a generic model'
 
 
-@skip_36
 def test_value_validation():
     T = TypeVar('T')
 
@@ -87,7 +83,6 @@ def test_value_validation():
     assert exc_info.value.errors() == [{'loc': ('__root__',), 'msg': 'sum too large', 'type': 'value_error'}]
 
 
-@skip_36
 def test_methods_are_inherited():
     class CustomGenericModel(GenericModel):
         def method(self):
@@ -103,7 +98,6 @@ def test_methods_are_inherited():
     assert instance.method() == 1
 
 
-@skip_36
 def test_config_is_inherited():
     class CustomGenericModel(GenericModel):
         class Config:
@@ -122,7 +116,6 @@ def test_config_is_inherited():
     assert str(exc_info.value) == '"Model[int]" is immutable and does not support item assignment'
 
 
-@skip_36
 def test_default_argument():
     T = TypeVar('T')
 
@@ -134,7 +127,6 @@ def test_default_argument():
     assert result.other is True
 
 
-@skip_36
 def test_default_argument_for_typevar():
     T = TypeVar('T')
 
@@ -151,7 +143,6 @@ def test_default_argument_for_typevar():
     assert result.data == 1
 
 
-@skip_36
 def test_classvar():
     T = TypeVar('T')
 
@@ -165,7 +156,6 @@ def test_classvar():
     assert 'other' not in Result.__fields__
 
 
-@skip_36
 def test_non_annotated_field():
     T = TypeVar('T')
 
@@ -180,7 +170,6 @@ def test_non_annotated_field():
     assert result.other is True
 
 
-@skip_36
 def test_must_inherit_from_generic():
     with pytest.raises(TypeError) as exc_info:
 
@@ -192,7 +181,6 @@ def test_must_inherit_from_generic():
     assert str(exc_info.value) == 'Type Result must inherit from typing.Generic before being parameterized'
 
 
-@skip_36
 def test_parameters_placed_on_generic():
     T = TypeVar('T')
     with pytest.raises(TypeError, match='Type parameters should be placed on typing.Generic, not GenericModel'):
@@ -201,7 +189,6 @@ def test_parameters_placed_on_generic():
             pass
 
 
-@skip_36
 def test_parameters_must_be_typevar():
     with pytest.raises(TypeError, match='Type GenericModel must inherit from typing.Generic before being '):
 
@@ -209,7 +196,6 @@ def test_parameters_must_be_typevar():
             pass
 
 
-@skip_36
 def test_subclass_can_be_genericized():
     T = TypeVar('T')
 
@@ -219,7 +205,6 @@ def test_subclass_can_be_genericized():
     Result[T]
 
 
-@skip_36
 def test_parameter_count():
     T = TypeVar('T')
     S = TypeVar('S')
@@ -237,7 +222,6 @@ def test_parameter_count():
     assert str(exc_info.value) == 'Too few parameters for Model; actual 1, expected 2'
 
 
-@skip_36
 def test_cover_cache():
     cache_size = len(_generic_types_cache)
     T = TypeVar('T')
@@ -251,7 +235,32 @@ def test_cover_cache():
     assert len(_generic_types_cache) == cache_size + 2
 
 
-@skip_36
+def test_cache_keys_are_hashable():
+    cache_size = len(_generic_types_cache)
+    T = TypeVar('T')
+    C = Callable[[str, Dict[str, Any]], Iterable[str]]
+
+    class MyGenericModel(GenericModel, Generic[T]):
+        t: T
+
+    # Callable's first params get converted to a list, which is not hashable.
+    # Make sure we can handle that special case
+    Simple = MyGenericModel[Callable[[int], str]]
+    assert len(_generic_types_cache) == cache_size + 2
+    # Nested Callables
+    MyGenericModel[Callable[[C], Iterable[str]]]
+    assert len(_generic_types_cache) == cache_size + 4
+    MyGenericModel[Callable[[Simple], Iterable[int]]]
+    assert len(_generic_types_cache) == cache_size + 6
+    MyGenericModel[Callable[[MyGenericModel[C]], Iterable[int]]]
+    assert len(_generic_types_cache) == cache_size + 10
+
+    class Model(BaseModel):
+        x: MyGenericModel[Callable[[C], Iterable[str]]] = Field(...)
+
+    assert len(_generic_types_cache) == cache_size + 10
+
+
 def test_generic_config():
     data_type = TypeVar('data_type')
 
@@ -267,7 +276,6 @@ def test_generic_config():
         result.data = 2
 
 
-@skip_36
 def test_enum_generic():
     T = TypeVar('T')
 
@@ -282,7 +290,6 @@ def test_enum_generic():
     Model[MyEnum](enum=2)
 
 
-@skip_36
 def test_generic():
     data_type = TypeVar('data_type')
     error_type = TypeVar('error_type')
@@ -337,7 +344,6 @@ def test_generic():
     ]
 
 
-@skip_36
 def test_alongside_concrete_generics():
     from pydantic.generics import GenericModel
 
@@ -352,7 +358,6 @@ def test_alongside_concrete_generics():
     assert model.metadata == {}
 
 
-@skip_36
 def test_complex_nesting():
     from pydantic.generics import GenericModel
 
@@ -366,7 +371,6 @@ def test_complex_nesting():
     assert model.item == item
 
 
-@skip_36
 def test_required_value():
     T = TypeVar('T')
 
@@ -378,7 +382,6 @@ def test_required_value():
     assert exc_info.value.errors() == [{'loc': ('a',), 'msg': 'field required', 'type': 'value_error.missing'}]
 
 
-@skip_36
 def test_optional_value():
     T = TypeVar('T')
 
@@ -389,7 +392,6 @@ def test_optional_value():
     assert model.dict() == {'a': 1}
 
 
-@skip_36
 def test_custom_schema():
     T = TypeVar('T')
 
@@ -400,7 +402,6 @@ def test_custom_schema():
     assert schema['properties']['a'].get('description') == 'Custom'
 
 
-@skip_36
 def test_child_schema():
     T = TypeVar('T')
 
@@ -419,7 +420,6 @@ def test_child_schema():
     }
 
 
-@skip_36
 def test_custom_generic_naming():
     T = TypeVar('T')
 
@@ -436,7 +436,6 @@ def test_custom_generic_naming():
     assert repr(MyModel[str](value=None)) == 'OptionalStrWrapper(value=None)'
 
 
-@skip_36
 def test_nested():
     AT = TypeVar('AT')
 
@@ -468,7 +467,6 @@ def test_nested():
     ]
 
 
-@skip_36
 def test_partial_specification():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -488,7 +486,6 @@ def test_partial_specification():
     ]
 
 
-@skip_36
 def test_partial_specification_with_inner_typevar():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -508,7 +505,6 @@ def test_partial_specification_with_inner_typevar():
     assert nested_resolved.b == [456]
 
 
-@skip_36
 def test_partial_specification_name():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -523,7 +519,6 @@ def test_partial_specification_name():
     assert concrete_model.__name__ == 'Model[int, BT][str]'
 
 
-@skip_36
 def test_partial_specification_instantiation():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -544,7 +539,6 @@ def test_partial_specification_instantiation():
     ]
 
 
-@skip_36
 def test_partial_specification_instantiation_bounded():
     AT = TypeVar('AT')
     BT = TypeVar('BT', bound=int)
@@ -569,7 +563,6 @@ def test_partial_specification_instantiation_bounded():
     ]
 
 
-@skip_36
 def test_typevar_parametrization():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -589,7 +582,6 @@ def test_typevar_parametrization():
     ]
 
 
-@skip_36
 def test_multiple_specification():
     AT = TypeVar('AT')
     BT = TypeVar('BT')
@@ -610,7 +602,6 @@ def test_multiple_specification():
     ]
 
 
-@skip_36
 def test_generic_subclass_of_concrete_generic():
     T = TypeVar('T')
     U = TypeVar('U')
@@ -632,7 +623,6 @@ def test_generic_subclass_of_concrete_generic():
     ConcreteSub(data=2, extra=3)
 
 
-@skip_36
 def test_generic_model_pickle(create_module):
     # Using create_module because pickle doesn't support
     # objects with <locals> in their __qualname__  (e. g. defined in function)
@@ -661,7 +651,6 @@ def test_generic_model_pickle(create_module):
         assert loaded == original
 
 
-@skip_36
 def test_generic_model_from_function_pickle_fail(create_module):
     @create_module
     def module():
@@ -690,7 +679,6 @@ def test_generic_model_from_function_pickle_fail(create_module):
             pickle.dumps(original)
 
 
-@skip_36
 def test_generic_model_redefined_without_cache_fail(create_module, monkeypatch):
 
     # match identity checker otherwise we never get to the redefinition check
@@ -727,6 +715,52 @@ def test_generic_model_redefined_without_cache_fail(create_module, monkeypatch):
         assert globals()['MyGeneric[Model]__'] is third_concrete
 
 
+def test_generic_model_caching_detect_order_of_union_args_basic(create_module):
+    # Basic variant of https://github.com/pydantic/pydantic/issues/4474
+    @create_module
+    def module():
+        from typing import Generic, TypeVar, Union
+
+        from pydantic.generics import GenericModel
+
+        t = TypeVar('t')
+
+        class Model(GenericModel, Generic[t]):
+            data: t
+
+        int_or_float_model = Model[Union[int, float]]
+        float_or_int_model = Model[Union[float, int]]
+
+        assert type(int_or_float_model(data='1').data) is int
+        assert type(float_or_int_model(data='1').data) is float
+
+
+@pytest.mark.skip(
+    reason="""
+Depends on similar issue in CPython itself: https://github.com/python/cpython/issues/86483
+Documented and skipped for possible fix later.
+"""
+)
+def test_generic_model_caching_detect_order_of_union_args_nested(create_module):
+    # Nested variant of https://github.com/pydantic/pydantic/issues/4474
+    @create_module
+    def module():
+        from typing import Generic, List, TypeVar, Union
+
+        from pydantic.generics import GenericModel
+
+        t = TypeVar('t')
+
+        class Model(GenericModel, Generic[t]):
+            data: t
+
+        int_or_float_model = Model[List[Union[int, float]]]
+        float_or_int_model = Model[List[Union[float, int]]]
+
+        assert type(int_or_float_model(data=['1']).data[0]) is int
+        assert type(float_or_int_model(data=['1']).data[0]) is float
+
+
 def test_get_caller_frame_info(create_module):
     @create_module
     def module():
@@ -772,7 +806,6 @@ def test_get_caller_frame_info_when_sys_getframe_undefined():
         sys._getframe = getframe
 
 
-@skip_36
 def test_iter_contained_typevars():
     T = TypeVar('T')
     T2 = TypeVar('T2')
@@ -786,7 +819,6 @@ def test_iter_contained_typevars():
     assert list(iter_contained_typevars(Optional[List[Union[str, Model[T], Callable[[T2, T], str]]]])) == [T, T2, T]
 
 
-@skip_36
 def test_nested_identity_parameterization():
     T = TypeVar('T')
     T2 = TypeVar('T2')
@@ -799,7 +831,6 @@ def test_nested_identity_parameterization():
     assert Model[T2] is not Model
 
 
-@skip_36
 def test_replace_types():
     T = TypeVar('T')
 
@@ -823,7 +854,6 @@ def test_replace_types():
         assert replace_types(list[Union[str, list, T]], {T: int}) == list[Union[str, list, int]]
 
 
-@skip_36
 def test_replace_types_with_user_defined_generic_type_field():
     """Test that using user defined generic types as generic model fields are handled correctly."""
 
@@ -847,7 +877,6 @@ def test_replace_types_with_user_defined_generic_type_field():
     assert replace_types(Model[T, VT, KT], {T: bool, KT: str, VT: int}) == Model[T, VT, KT][bool, int, str]
 
 
-@skip_36
 def test_replace_types_identity_on_unchanged():
     T = TypeVar('T')
     U = TypeVar('U')
@@ -856,7 +885,6 @@ def test_replace_types_identity_on_unchanged():
     assert replace_types(type_, {T: int}) is type_
 
 
-@skip_36
 def test_deep_generic():
     T = TypeVar('T')
     S = TypeVar('S')
@@ -888,7 +916,6 @@ def test_deep_generic():
     assert inner_model.__concrete__ is True
 
 
-@skip_36
 def test_deep_generic_with_inner_typevar():
     T = TypeVar('T')
 
@@ -906,7 +933,6 @@ def test_deep_generic_with_inner_typevar():
     assert InnerModel[int](a=['1']).a == [1]
 
 
-@skip_36
 def test_deep_generic_with_referenced_generic():
     T = TypeVar('T')
     R = TypeVar('R')
@@ -928,7 +954,6 @@ def test_deep_generic_with_referenced_generic():
     assert InnerModel[int](a={'a': 1}).a.a == 1
 
 
-@skip_36
 def test_deep_generic_with_referenced_inner_generic():
     T = TypeVar('T')
 
@@ -952,7 +977,6 @@ def test_deep_generic_with_referenced_inner_generic():
     assert (InnerModel[int].__fields__['a'].sub_fields[0].sub_fields[0].outer_type_.__fields__['a'].outer_type_) == int
 
 
-@skip_36
 def test_deep_generic_with_multiple_typevars():
     T = TypeVar('T')
     U = TypeVar('U')
@@ -970,7 +994,6 @@ def test_deep_generic_with_multiple_typevars():
     assert ConcreteInnerModel(data=['1'], extra='2').dict() == {'data': [1.0], 'extra': 2}
 
 
-@skip_36
 def test_deep_generic_with_multiple_inheritance():
     K = TypeVar('K')
     V = TypeVar('V')
@@ -998,7 +1021,6 @@ def test_deep_generic_with_multiple_inheritance():
     }
 
 
-@skip_36
 def test_generic_with_referenced_generic_type_1():
     T = TypeVar('T')
 
@@ -1013,7 +1035,6 @@ def test_generic_with_referenced_generic_type_1():
     ReferenceModel[int]
 
 
-@skip_36
 def test_generic_with_referenced_nested_typevar():
     T = TypeVar('T')
 
@@ -1029,7 +1050,6 @@ def test_generic_with_referenced_nested_typevar():
     ReferenceModel[int]
 
 
-@skip_36
 def test_generic_with_callable():
     T = TypeVar('T')
 
@@ -1041,7 +1061,6 @@ def test_generic_with_callable():
     Model.__concrete__ is False
 
 
-@skip_36
 def test_generic_with_partial_callable():
     T = TypeVar('T')
     U = TypeVar('U')
@@ -1057,7 +1076,6 @@ def test_generic_with_partial_callable():
     Model[str, int].__concrete__ is False
 
 
-@skip_36
 def test_generic_recursive_models(create_module):
     @create_module
     def module():
@@ -1081,7 +1099,6 @@ def test_generic_recursive_models(create_module):
     assert result == Model1(ref=Model2(ref=Model1(ref=Model2(ref='123'))))
 
 
-@skip_36
 def test_generic_enum():
     T = TypeVar('T')
 
@@ -1099,7 +1116,6 @@ def test_generic_enum():
     assert m.my_gen.some_field is SomeStringEnum.A
 
 
-@skip_36
 def test_generic_literal():
     FieldType = TypeVar('FieldType')
     ValueType = TypeVar('ValueType')
@@ -1112,7 +1128,6 @@ def test_generic_literal():
     assert m.dict() == {'field': {'foo': 'x'}}
 
 
-@skip_36
 def test_generic_enums():
     T = TypeVar('T')
 
@@ -1132,7 +1147,6 @@ def test_generic_enums():
     assert set(Model.schema()['definitions']) == {'EnumA', 'EnumB', 'GModel_EnumA_', 'GModel_EnumB_'}
 
 
-@skip_36
 def test_generic_with_user_defined_generic_field():
     T = TypeVar('T')
 
@@ -1150,7 +1164,6 @@ def test_generic_with_user_defined_generic_field():
         model = Model[int](field=['a'])
 
 
-@skip_36
 def test_generic_annotated():
     T = TypeVar('T')
 
@@ -1160,7 +1173,6 @@ def test_generic_annotated():
     SomeGenericModel[str](the_alias='qwe')
 
 
-@skip_36
 def test_generic_subclass():
     T = TypeVar('T')
 
@@ -1176,7 +1188,6 @@ def test_generic_subclass():
     assert not issubclass(B[int], A[str])
 
 
-@skip_36
 def test_generic_subclass_with_partial_application():
     T = TypeVar('T')
     S = TypeVar('S')
@@ -1193,7 +1204,6 @@ def test_generic_subclass_with_partial_application():
     assert not issubclass(PartiallyAppliedB[str], A[int])
 
 
-@skip_36
 def test_multilevel_generic_binding():
     T = TypeVar('T')
     S = TypeVar('S')
@@ -1209,7 +1219,6 @@ def test_multilevel_generic_binding():
     assert not issubclass(B[str], A[str, int])
 
 
-@skip_36
 def test_generic_subclass_with_extra_type():
     T = TypeVar('T')
     S = TypeVar('S')
@@ -1226,7 +1235,6 @@ def test_generic_subclass_with_extra_type():
     assert not issubclass(B[int, str], A[int])
 
 
-@skip_36
 def test_multi_inheritance_generic_binding():
     T = TypeVar('T')
 
@@ -1246,7 +1254,6 @@ def test_multi_inheritance_generic_binding():
     assert not issubclass(C[float], A[str])
 
 
-@skip_36
 def test_parse_generic_json():
     T = TypeVar('T')
 
