@@ -19,6 +19,7 @@ from typing import (
 from uuid import UUID, uuid4
 
 import pytest
+from typing_extensions import Annotated
 
 from pydantic import (
     BaseConfig,
@@ -36,7 +37,7 @@ from pydantic import (
     root_validator,
     validator,
 )
-from pydantic.typing import Literal
+from pydantic.typing import Final, Literal
 
 
 def test_success():
@@ -585,12 +586,14 @@ def test_const_list():
         'properties': {
             'a': {
                 'const': [SubModel(b=1), SubModel(b=2), SubModel(b=3)],
+                'default': [{'b': 1}, {'b': 2}, {'b': 3}],
                 'items': {'$ref': '#/definitions/SubModel'},
                 'title': 'A',
                 'type': 'array',
             },
             'b': {
                 'const': [{'b': 4}, {'b': 5}, {'b': 6}],
+                'default': [{'b': 4}, {'b': 5}, {'b': 6}],
                 'items': {'$ref': '#/definitions/SubModel'},
                 'title': 'B',
                 'type': 'array',
@@ -978,18 +981,20 @@ def test_type_type_validation_fails_for_basic_type():
     ]
 
 
-def test_bare_type_type_validation_success():
+@pytest.mark.parametrize('bare_type', [type, Type])
+def test_bare_type_type_validation_success(bare_type):
     class ArbitraryClassAllowedModel(BaseModel):
-        t: Type
+        t: bare_type
 
     arbitrary_type_class = ArbitraryType
     m = ArbitraryClassAllowedModel(t=arbitrary_type_class)
     assert m.t == arbitrary_type_class
 
 
-def test_bare_type_type_validation_fails():
+@pytest.mark.parametrize('bare_type', [type, Type])
+def test_bare_type_type_validation_fails(bare_type):
     class ArbitraryClassAllowedModel(BaseModel):
-        t: Type
+        t: bare_type
 
     arbitrary_type = ArbitraryType()
     with pytest.raises(ValidationError) as exc_info:
@@ -1931,7 +1936,6 @@ def test_default_factory_parse():
     assert repr(parsed) == 'Outer(inner_1=Inner(val=0), inner_2=Inner(val=0))'
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason='field constraints are set but not enforced with python 3.6')
 def test_none_min_max_items():
     # None default
     class Foo(BaseModel):
@@ -2103,7 +2107,6 @@ def test_typing_coercion_dict():
     assert repr(m) == "Model(x={'one': 1, 'two': 2})"
 
 
-@pytest.mark.skipif(sys.version_info < (3, 7), reason='generic classes need 3.7')
 def test_typing_non_coercion_of_dict_subclasses():
     KT = TypeVar('KT')
     VT = TypeVar('VT')
@@ -2222,3 +2225,76 @@ def test_new_union_origin():
         'properties': {'x': {'title': 'X', 'anyOf': [{'type': 'integer'}, {'type': 'string'}]}},
         'required': ['x'],
     }
+
+
+def test_annotated_class():
+    class PydanticModel(BaseModel):
+        foo: str = '123'
+
+    PydanticAlias = Annotated[PydanticModel, 'bar baz']
+
+    pa = PydanticAlias()
+    assert isinstance(pa, PydanticModel)
+    pa.__doc__ = 'qwe'
+    assert repr(pa) == "PydanticModel(foo='123')"
+    assert pa.__doc__ == 'qwe'
+
+
+@pytest.mark.parametrize(
+    'ann',
+    [Final, Final[int]],
+    ids=['no-arg', 'with-arg'],
+)
+@pytest.mark.parametrize(
+    'value',
+    [None, Field(...)],
+    ids=['none', 'field'],
+)
+def test_final_field_decl_withou_default_val(ann, value):
+    class Model(BaseModel):
+        a: ann
+
+        if value is not None:
+            a = value
+
+    Model.update_forward_refs(ann=ann)
+
+    assert 'a' not in Model.__class_vars__
+    assert 'a' in Model.__fields__
+
+    assert Model.__fields__['a'].final
+
+
+@pytest.mark.parametrize(
+    'ann',
+    [Final, Final[int]],
+    ids=['no-arg', 'with-arg'],
+)
+def test_final_field_decl_with_default_val(ann):
+    class Model(BaseModel):
+        a: ann = 10
+
+    Model.update_forward_refs(ann=ann)
+
+    assert 'a' in Model.__class_vars__
+    assert 'a' not in Model.__fields__
+
+
+def test_final_field_reassignment():
+    class Model(BaseModel):
+        a: Final[int]
+
+    obj = Model(a=10)
+
+    with pytest.raises(
+        TypeError,
+        match=r'^"Model" object "a" field is final and does not support reassignment$',
+    ):
+        obj.a = 20
+
+
+def test_field_by_default_is_not_final():
+    class Model(BaseModel):
+        a: int
+
+    assert not Model.__fields__['a'].final
