@@ -1,10 +1,11 @@
-from typing import Dict, List, Mapping
+import json
+from typing import Dict, List, Mapping, Union
 
 import pytest
 
 from pydantic import BaseModel, ValidationError
 from pydantic.dataclasses import dataclass
-from pydantic.tools import parse_file_as, parse_obj_as
+from pydantic.tools import parse_file_as, parse_obj_as, parse_raw_as, schema_json_of, schema_of
 
 
 @pytest.mark.parametrize('obj,type_,parsed', [('1', int, 1), (['1'], List[int], [1])])
@@ -76,3 +77,44 @@ def test_parse_file_as(tmp_path):
     p = tmp_path / 'test.json'
     p.write_text('{"1": "2"}')
     assert parse_file_as(Dict[int, int], p) == {1: 2}
+
+
+def test_parse_file_as_json_loads(tmp_path):
+    def custom_json_loads(*args, **kwargs):
+        data = json.loads(*args, **kwargs)
+        data[1] = 99
+        return data
+
+    p = tmp_path / 'test_json_loads.json'
+    p.write_text('{"1": "2"}')
+    assert parse_file_as(Dict[int, int], p, json_loads=custom_json_loads) == {1: 99}
+
+
+def test_raw_as():
+    class Item(BaseModel):
+        id: int
+        name: str
+
+    item_data = '[{"id": 1, "name": "My Item"}]'
+    items = parse_raw_as(List[Item], item_data)
+    assert items == [Item(id=1, name='My Item')]
+
+
+def test_schema():
+    assert schema_of(Union[int, str], title='IntOrStr') == {
+        'title': 'IntOrStr',
+        'anyOf': [{'type': 'integer'}, {'type': 'string'}],
+    }
+    assert schema_json_of(Union[int, str], title='IntOrStr', indent=2) == (
+        '{\n'
+        '  "title": "IntOrStr",\n'
+        '  "anyOf": [\n'
+        '    {\n'
+        '      "type": "integer"\n'
+        '    },\n'
+        '    {\n'
+        '      "type": "string"\n'
+        '    }\n'
+        '  ]\n'
+        '}'
+    )
